@@ -4,14 +4,17 @@ from pypop7.optimizers.es.ssaes import SSAES
 from pypop7.optimizers.es.dsaes import DSAES
 from pypop7.optimizers.es.csaes import CSAES
 from pypop7.optimizers.es.saes import SAES
+from pypop7.optimizers.es.sepcmaes import SEPCMAES
 from pypop7.optimizers.es.maes import MAES
 from pypop7.optimizers.es.fmaes import FMAES
 from pypop7.optimizers.es.r1es import R1ES
+from pypop7.optimizers.es.rmes import RMES
 from pypop7.optimizers.es.lmcmaes import LMCMAES
 from pypop7.optimizers.es.lmcma import LMCMA
+from pypop7.optimizers.es.lmmaes import LMMAES
 
 
-__all__ = [ES,
+__all__ = [ES,  # base (abstract) class
            RES, SSAES, DSAES, CSAES, SAES,  # representative ES versions during development
            MAES, FMAES,  # modern ES versions often with state-of-the-art performance
-           R1ES, LMCMAES, LMCMA]  # especially for large-scale black-box optimization
+           SEPCMAES, R1ES, RMES, LMCMAES, LMCMA, LMMAES]  # especially for large-scale black-box optimization
